export declare const DefaultBufferLength = 1024;
export declare function isTagged(type: number): boolean;
export declare function grammarID(type: number): number;
export declare function termID(type: number): number;
export declare function allocateGrammarID(): number;
export interface ChangedRange {
    fromA: number;
    toA: number;
    fromB: number;
    toB: number;
}
export declare type EnterFunc<T> = (type: number, start: number, end: number) => T | false | undefined;
export declare type LeaveFunc = (type: number, start: number, end: number) => void;
declare class Iteration<T> {
    readonly enter: EnterFunc<T>;
    readonly leave: LeaveFunc | undefined;
    result: T | undefined;
    constructor(enter: EnterFunc<T>, leave: LeaveFunc | undefined);
    readonly done: boolean;
    doEnter(type: number, start: number, end: number): boolean;
}
export declare abstract class Subtree {
    abstract parent: Subtree | null;
    abstract type: number;
    abstract start: number;
    abstract end: number;
    readonly depth: number;
    readonly root: Tree;
    abstract toString(tags?: TagMap<any>): string;
    abstract iterate<T = any>(from: number, to: number, enter: EnterFunc<T>, leave?: LeaveFunc): T | undefined;
    resolve(pos: number, side?: -1 | 0 | 1): Subtree;
    abstract resolveAt(pos: number): Subtree;
    abstract childBefore(pos: number): Subtree | null;
    abstract childAfter(pos: number): Subtree | null;
    readonly firstChild: Subtree | null;
    readonly lastChild: Subtree | null;
}
export declare class Tree extends Subtree {
    readonly children: (Tree | TreeBuffer)[];
    readonly positions: number[];
    readonly length: number;
    readonly type: number;
    parent: null;
    constructor(children: (Tree | TreeBuffer)[], positions: number[], length: number, type?: number);
    readonly start: number;
    toString(tags?: TagMap<any>): string;
    readonly end: number;
    private partial;
    applyChanges(changes: readonly ChangedRange[]): Tree;
    cut(at: number): Tree;
    static empty: Tree;
    iterate<T = any>(from: number, to: number, enter: EnterFunc<T>, leave?: LeaveFunc): T | undefined;
    iterInner<T>(from: number, to: number, offset: number, iter: Iteration<T>): void;
    resolveAt(pos: number, side?: -1 | 0 | 1): Subtree;
    childBefore(pos: number): Subtree | null;
    childAfter(pos: number): Subtree | null;
    findChild(pos: number, side: number, start: number, parent: Subtree): Subtree | null;
    resolveInner(pos: number, start: number, parent: Subtree): Subtree;
    append(other: Tree): Tree;
    balance(maxBufferLength?: number): Tree;
    static build(buffer: BufferCursor | readonly number[], grammarID: number, maxBufferLength?: number, reused?: Tree[]): Tree;
}
export declare class TreeBuffer {
    readonly buffer: Uint16Array;
    readonly length: number;
    readonly grammarID: number;
    constructor(buffer: Uint16Array, length: number, grammarID: number);
    toString(tags?: TagMap<any>): string;
    childToString(index: number, parts: string[], tags?: TagMap<any>): number;
    cut(at: number): TreeBuffer;
    iterInner<T>(from: number, to: number, offset: number, iter: Iteration<T>): void;
    iterChild<T>(from: number, to: number, offset: number, index: number, iter: Iteration<T>): number;
    private parentNodesByEnd;
    iterRev<T>(from: number, to: number, offset: number, startIndex: number, endIndex: number, iter: Iteration<T>): void;
    findIndex(pos: number, side: number, start: number, from: number, to: number): number;
}
export interface BufferCursor {
    pos: number;
    type: number;
    start: number;
    end: number;
    size: number;
    next(): void;
    fork(): BufferCursor;
}
export declare class TagMap<T> {
    readonly grammars: {
        readonly [id: number]: readonly (T | null)[];
    };
    constructor(grammars: {
        readonly [id: number]: readonly (T | null)[];
    });
    get(type: number): T | null;
    static single<T>(grammarID: number, values: readonly (T | null)[]): TagMap<T>;
    static combine<T>(maps: TagMap<T>[]): TagMap<T>;
    static empty: TagMap<any>;
}
export {};
